/*
 * ====================================================================
 *  Container.cpp
 *
 *  Window-owning container control class CAmarettoContainer
 *
 * Copyright (c) 2005-2009 Nokia Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

#include "Container.h"

CAmarettoContainer* CAmarettoContainer::NewL(const TRect& aRect)
{
  CAmarettoContainer *self = new (ELeave) CAmarettoContainer;
  CleanupStack::PushL(self);
  self->ConstructL(aRect);
  CleanupStack::Pop();
  return self;
} 

void CAmarettoContainer::ConstructL(const TRect& aRect)
{
  CreateWindowL();
  iTop = NULL;
  SetRect(aRect);
  iPointerForwardDisabled = EFalse;
  ActivateL();
}

CAmarettoContainer::~CAmarettoContainer()
{
  if (iDecoratedTabGroup) {
    GetNaviPane()->Pop(iDecoratedTabGroup);
    delete iDecoratedTabGroup;
  }
  RemoveNaviKeys();
}

CAknNavigationControlContainer* CAmarettoContainer::GetNaviPane() const
{
  CEikStatusPane* sp = 
    (STATIC_CAST(CAknAppUi*, iEikonEnv->EikAppUi()))->StatusPane();
  return (STATIC_CAST(CAknNavigationControlContainer*,
                      sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi))));
}

void CAmarettoContainer::EnableTabsL(const CDesCArray* aTabTexts,
                                     CAmarettoCallback* aFunc)
{
  if (iDecoratedTabGroup) {
    GetNaviPane()->Pop(iDecoratedTabGroup);
    delete iDecoratedTabGroup;
  }
  if (aTabTexts) {
    iDecoratedTabGroup = GetNaviPane()->CreateTabGroupL();
    iTabGroup = STATIC_CAST(CAknTabGroup*,
                            iDecoratedTabGroup->DecoratedControl());
  
    for (int i = 0; i < aTabTexts->Count(); i++)
      iTabGroup->AddTabL(i, (*aTabTexts)[i]);
  
    GetNaviPane()->PushL(*iDecoratedTabGroup);
    iTabCallback = aFunc;
    iTabGroup->SetActiveTabByIndex(0);
    
    if (iTop)
      iTop->SetFocus(EFalse);
    iTabGroup->SetFocus(ETrue);

    //  Refresh();
  }
  else {
    iDecoratedTabGroup = NULL;
    iTabGroup = NULL;
    iTabCallback = NULL;
    if (iTop)
      iTop->SetFocus(ETrue);
  }
}

void CAmarettoContainer::SetActiveTab(TInt aIndex)
{
  if (iTabGroup && (aIndex >= 0) && (aIndex < iTabGroup->TabCount()))
    iTabGroup->SetActiveTabByIndex(aIndex);
}

TInt CAmarettoContainer::SetComponentL(CCoeControl* aComponent,
                                       CAmarettoCallback* aEventHandler, char *aControlName)
{
  if (iTop) {
      iTop->MakeVisible(EFalse);
      iTop->SetFocus(EFalse);
      if (!strcmp(iTopControlName, "canvas")){
          if (virtual_keys) {
              virtual_keys->MakeVisible(EFalse);
              naviKeysIsVisible = EFalse;
          }
      }
  }
  iTop = aComponent;

  if (aControlName != NULL)
      iTopControlName = aControlName;
  
  if (iTop) {
      if (!strcmp(iTopControlName, "canvas")){
          if (virtual_keys) {
              virtual_keys->MakeVisible(ETrue);
               naviKeysIsVisible = ETrue;
          }
      }
    iTop->MakeVisible(ETrue);
  }
  iEventCallback = aEventHandler;

  if (aComponent && !iTabGroup)
    aComponent->SetFocus(ETrue);

  Refresh();

  return KErrNone;
}

void CAmarettoContainer::SizeChanged() 
{
    if (iTop) { 
        TRect containerRect = Rect();
        TInt a  = containerRect.iTl.iX;
        TInt b = containerRect.iTl.iY;
        TInt c = containerRect.iBr.iX;
        TInt d = containerRect.iBr.iY;
        TInt width = c - a;
        TInt height = d - b;
        if (strcmp(iTopControlName, "canvas") || !virtual_d_pad_flag || (width > height)){
            iTop->SetRect(Rect());
        }
        else {
            TInt e = 4 * d / 5;
            // User Canvas rect size relative to Container 
            TRect canvas_rect = TRect(a, b, c, e);
            iTop->SetRect(canvas_rect);
            TRect virtual_key_rect = TRect(0, e, c, d);
            if (!virtual_keys)
                CreateNaviKeys(iAppui);
            virtual_keys->SetRect(virtual_key_rect);
            virtual_keys->CalculateAndDrawGraphic(ETrue, navi_keys_gc);
        }
    }
}

void CAmarettoContainer::Draw(const TRect& aRect) const
{
  CWindowGc& gc = SystemGc();
  gc.SetPenStyle(CGraphicsContext::ENullPen);
  gc.SetBrushColor(KRgbWhite);
  gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
  gc.DrawRect(aRect);
  if (virtual_keys && naviKeysIsVisible)
      virtual_keys->CalculateAndDrawGraphic(ETrue, navi_keys_gc);
}

void CAmarettoContainer::RefreshNaviKeys()
{
    if (virtual_keys && naviKeysIsVisible) {
        virtual_keys->CalculateAndDrawGraphic(ETrue, navi_keys_gc);
    }
}

TKeyResponse
CAmarettoContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
  if (iTabGroup && ((aKeyEvent.iCode == EKeyLeftArrow) ||
                    (aKeyEvent.iCode == EKeyRightArrow))) {
    
    TInt active = iTabGroup->ActiveTabIndex();
    TInt count = iTabGroup->TabCount();
    
    if ((aKeyEvent.iCode == EKeyLeftArrow) && (active > 0))
      active--;
    else if ((aKeyEvent.iCode == EKeyRightArrow) && ((active + 1) < count))
      active++;
    else
      return EKeyWasConsumed;
    
    iTabGroup->SetActiveTabByIndex(active);
    iTabCallback->Call((void*)active);

    Refresh();
    
    return EKeyWasConsumed;
  } 
  
  if (iTop) {
    if ((aType == EEventKey) && iEventCallback) {
      SAmarettoEventInfo event_info;
      event_info.iType = SAmarettoEventInfo::EKey;
      event_info.iKeyEvent = aKeyEvent;
      iEventCallback->Call((void*)&event_info);
    }
    return iTop->OfferKeyEventL(aKeyEvent, aType);
  }
  else
    return EKeyWasNotConsumed;
}

void CAmarettoContainer::DisablePointerForwarding(TBool aValue)
{
	iPointerForwardDisabled = aValue;
}

TInt CAmarettoContainer::CreateNaviKeys(CAmarettoAppUi* aAppui)
{
    if (!virtual_keys) {
        TRect my_rect = Rect();
        TInt a  = my_rect.iTl.iX;
        TInt b = my_rect.iTl.iY;
        TInt c = my_rect.iBr.iX;
        TInt d = my_rect.iBr.iY;
        TInt e = 4 * d / 5;
        
        // User Canvas rect size relative to Container 
        TRect canvas_rect = TRect(a, b, c, e);
        // Navi keys rect size relative to Container
        TRect virtual_key_rect = TRect(0, e, c, d);
        virtual_keys = new VirtualNaviKeys();
        if (virtual_keys == NULL)
            return -1;
        TRAPD(error, {
                virtual_keys->ConstructL(virtual_key_rect, this, aAppui);
                navi_keys_gc = virtual_keys->ControlEnv()->CreateGcL();
        });
        if (error != KErrNone)
            return error;
        navi_keys_gc->Activate(*virtual_keys->DrawableWindow());
    }
}

void CAmarettoContainer::RemoveNaviKeys()
{
    delete virtual_keys;
    virtual_keys = NULL;
    if (navi_keys_gc)
        navi_keys_gc->Deactivate();
    delete navi_keys_gc;
    navi_keys_gc = NULL;
}

void CAmarettoContainer::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
    // Check if touch is enabled or not
    if(!touch_enabled_flag) {
        return;
    }
    if (iTop && iEventCallback) {
          SAmarettoEventInfo event_info;
          event_info.iType = SAmarettoEventInfo::EPointer;
          memset(&event_info.iKeyEvent, 0, sizeof(event_info.iKeyEvent));
          event_info.iPointerEvent = aPointerEvent;
          iEventCallback->Call((void*)&event_info);
        }
    // Call base class method, that forwards pointer event to the right child
	// component. We disable this for Canvas object.
    if(!iPointerForwardDisabled)
    	CCoeControl::HandlePointerEventL(aPointerEvent);
}




void drawKeys(CWindowGc *gc, TRect keyRect, CArrayFix<TPoint>* polygonPoints) {
    gc->SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
    gc->SetBrushColor(TRgb(140, 140, 140));
    gc->SetPenColor(TRgb(140, 140, 140));
    gc->DrawRoundRect(keyRect, TSize(3,3));
    //inner fill
    gc->SetBrushStyle((CGraphicsContext::TBrushStyle) 1 );
    gc->SetBrushColor(TRgb(196,196,196)); 
    gc->SetPenColor(TRgb(196,196,196)); 
    keyRect.Shrink(3,3);
    gc->DrawRoundRect(keyRect, TSize(3,3));
    gc->SetBrushColor(TRgb(0,0,0));
    gc->SetPenColor(TRgb(0,0,0));
    if (polygonPoints!=NULL)
        gc->DrawPolygon(polygonPoints, (CGraphicsContext::TFillRule) 0);
}

VirtualNaviKeys::VirtualNaviKeys() {}

VirtualNaviKeys::~VirtualNaviKeys()
{
    delete left_triangle;
    left_triangle = NULL;
    delete right_triangle;
    right_triangle = NULL;
    delete up_triangle;
    up_triangle = NULL;
    delete down_triangle;
    down_triangle = NULL;
    delete center_rect;
    center_rect = NULL;
}
void VirtualNaviKeys::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
  TKeyEvent keyEvent;
  keyEvent.iModifiers = EModifierKeypad;
  keyEvent.iRepeats   = 0;
   
  if (left.Contains(aPointerEvent.iPosition)){
    keyEvent.iCode      = EKeyLeftArrow;
    keyEvent.iScanCode  = EStdKeyLeftArrow;
  } else if (right.Contains(aPointerEvent.iPosition)){
    keyEvent.iCode      = EKeyRightArrow;
    keyEvent.iScanCode  = EStdKeyRightArrow;
  } else if (up.Contains(aPointerEvent.iPosition)){
      keyEvent.iCode      = EKeyUpArrow;
      keyEvent.iScanCode  = EStdKeyUpArrow;
  } else if (down.Contains(aPointerEvent.iPosition)){
      keyEvent.iCode      = EKeyDownArrow;
      keyEvent.iScanCode  = EStdKeyDownArrow;
  } else if (center.Contains(aPointerEvent.iPosition)){
      keyEvent.iCode      = EKeyDevice3;
      keyEvent.iScanCode  = EStdKeyDevice3;
  } else if (myAppui->iScreenMode == 2 && topright.Contains(aPointerEvent.iPosition)){
      myAppui->HandleCommandL(EAknSoftkeyExit);
  } else if (myAppui->iScreenMode == 2 && topleft.Contains(aPointerEvent.iPosition)){
      myAppui->ProcessCommandL(EAknSoftkeyOptions);
}

  if (aPointerEvent.iType == TPointerEvent::EButton1Up) {
      CONST_CAST(CCoeControl*, myParent)->OfferKeyEventL(keyEvent, EEventKeyUp);
  }
  
  if (aPointerEvent.iType == TPointerEvent::EButton1Down) {
      CONST_CAST(CCoeControl*, myParent)->OfferKeyEventL(keyEvent, EEventKeyDown);
      CONST_CAST(CCoeControl*, myParent)->OfferKeyEventL(keyEvent, EEventKey);
  }
}

void VirtualNaviKeys::ConstructL(const TRect& aRect,
                const CCoeControl* aParent, CAmarettoAppUi* aAppui)
{
    CreateWindowL(aParent);
    
    myParent = aParent;
    myAppui = aAppui;
    SetRect(aRect);
    ActivateL();
    CalculateAndDrawGraphic(EFalse, NULL);
   
}

void VirtualNaviKeys::CalculateAndDrawGraphic(TBool aDrawKeys, CWindowGc *gc)
{
    TRect container_rect = myAppui->iContainer->Rect();
    TInt a  = container_rect.iTl.iX;
    TInt b = container_rect.iTl.iY;
    TInt c = container_rect.iBr.iX;
    TInt d = container_rect.iBr.iY;
    TInt e = 4 * d / 5;

    // Actual size of the control
    TRect iSize = TRect(0, 0, c, d - e);
    SetRect(TRect(0, e, c, d));
    tlX  = iSize.iTl.iX;
    tlY = iSize.iTl.iY;
    brX = iSize.iBr.iX;
    brY = iSize.iBr.iY;
    left.SetRect(tlX, brY/3, brX/3, 2*brY/3);
    right.SetRect(2*brX/3, brY/3, brX, 2*brY/3);
    center.SetRect(brX/3, brY/3, 2*brX/3, 2*brY/3);
    up.SetRect(brX/3, tlY, 2*brX/3, brY/3);
    down.SetRect(brX/3, 2*brY/3, 2*brX/3, brY);
    topleft.SetRect(tlX, tlY, brX/3, brY/3);
    topright.SetRect(2*brX/3, tlX, brX, brY/3);
    
    TInt height = brY/3;
    TInt width = brX/3;
    
    left_triangle = new CArrayFixFlat<TPoint>(12);
    CleanupStack::PushL(left_triangle);
    TPoint point_1 = TPoint(2 * width / 3, (height / 3) + height);
    TPoint point_2 = TPoint(2 * width / 3, (2 * height / 3) + height);
    TPoint point_3 = TPoint(width / 3, (((height / 3) + (height / (2*3)))) + height);
    
    left_triangle->AppendL(point_1);
    left_triangle->AppendL(point_2);
    left_triangle->AppendL(point_3);
    
    CleanupStack::Pop(left_triangle);
    
    right_triangle = new CArrayFixFlat<TPoint>(12);
    CleanupStack::PushL(right_triangle);
    point_1 = TPoint((width / 3) + (2 * width), (height / 3) + height);
    point_2 = TPoint((width / 3) + (2 * width), (2 * height / 3) + height);
    point_3 = TPoint((2 * width / 3) + (2 * width), (((height / 3) + (height / (2*3)))) + height);
    
    right_triangle->AppendL(point_1);
    right_triangle->AppendL(point_2);
    right_triangle->AppendL(point_3);
    
    CleanupStack::Pop(right_triangle);
    
    up_triangle = new CArrayFixFlat<TPoint>(12);
    CleanupStack::PushL(up_triangle);
    point_1 = TPoint((width / 3) + (width), (2 * height / 3));
    point_2 = TPoint((2 * width / 3) + (width), (2 * height / 3));
    point_3 = TPoint(((width / 3) + (width / 6)) + (width), ((height / 3)));
    
    up_triangle->AppendL(point_1);
    up_triangle->AppendL(point_2);
    up_triangle->AppendL(point_3);
    
    CleanupStack::Pop(up_triangle);
    
    down_triangle = new CArrayFixFlat<TPoint>(12);
    CleanupStack::PushL(down_triangle);
    point_1 = TPoint((width / 3) + (width), (height / 3) + (2 * height));
    point_2 = TPoint((2 * width / 3) + (width), (height / 3) + (2 * height));
    point_3 = TPoint(((width / 3) + (width / 6)) + (width), (2 * height / 3) + (2 * height));
    
    down_triangle->AppendL(point_1);
    down_triangle->AppendL(point_2);
    down_triangle->AppendL(point_3);
    
    CleanupStack::Pop(down_triangle);  
    
    center_rect = new CArrayFixFlat<TPoint>(12);
    CleanupStack::PushL(center_rect);
    point_1 = TPoint((width / 3) + (width), (height / 3) + (height));
    point_2 = TPoint((2 * width / 3) + (width), (height / 3) + (height));
    point_3 = TPoint((width / 3) + (width), (2 * height / 3) + (height));
    TPoint point_4 = TPoint((2 * width / 3) + (width), (2 * height / 3) + (height));
    
    center_rect->AppendL(point_1);
    center_rect->AppendL(point_2);
    center_rect->AppendL(point_4);
    center_rect->AppendL(point_3);  
    
    CleanupStack::Pop(center_rect);  

    if (aDrawKeys){
        if (gc){
        gc->SetBrushColor(TRgb(127, 127, 127));
        gc->Clear();
        drawKeys(gc, left, left_triangle);
        drawKeys(gc, right, right_triangle);
        drawKeys(gc, center, center_rect);
        drawKeys(gc, up, up_triangle);
        drawKeys(gc, down, down_triangle);

        if (myAppui->iScreenMode == 2)
        {
            drawKeys(gc, topleft, NULL);
            drawKeys(gc, topright, NULL);
            const CFont* f = CEikonEnv::Static()->NormalFont(); 
            gc->UseFont(f); 
            gc->SetPenStyle( CGraphicsContext::ESolidPen ); 
            gc->SetPenColor(KRgbBlack); 
            gc->SetBrushStyle(CGraphicsContext::ENullBrush);
            gc->SetPenSize(TSize(1, 1));
            gc->DrawText(_L("Options"), topleft, (topleft.Height()/2 + f->HeightInPixels()/3), CGraphicsContext::ECenter, 0); 
            gc->DrawText(_L("Exit"), topright, (topleft.Height()/2 + f->HeightInPixels()/3), CGraphicsContext::ECenter, 0);
            gc->DiscardFont();
        }
        }
    }
    delete left_triangle;
    left_triangle = NULL;
    delete right_triangle;
    right_triangle = NULL;
    delete up_triangle;
    up_triangle = NULL;
    delete down_triangle;
    down_triangle = NULL;
    delete center_rect;
    center_rect = NULL;
}

